@@ -30,7 +30,7 @@ from reportlab.lib.colors import gray, lightgrey
 from reportlab.lib.rl_accel import fp_str
 from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
 from reportlab.lib.styles import _baseFontName
-from reportlab.lib.utils import strTypes, rl_safe_exec
+from reportlab.lib.utils import strTypes, rl_safe_exec, annotateException
 from reportlab.lib.abag import ABag
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase.pdfmetrics import stringWidth
@@ -691,10 +691,8 @@ class KeepTogether(_ContainerSpace,Flowable):
             #cache these on the class
             from reportlab.platypus.doctemplate import NullActionFlowable
             from reportlab.platypus.doctemplate import FrameBreak
-            from reportlab.lib.utils import annotateException
             KeepTogether.NullActionFlowable = NullActionFlowable
             KeepTogether.FrameBreak = FrameBreak
-            KeepTogether.annotateException = annotateException
 
         if not flowables:
             flowables = [self.NullActionFlowable()]
@@ -713,7 +711,7 @@ class KeepTogether(_ContainerSpace,Flowable):
         try:
             W,H = _listWrapOn(self._content,aW,self.canv,dims=dims)
         except:
-            self.annotateException('\nraised by class %s(%s)@0x%8.8x wrap\n' % (self.__class__.__name__,self.__class__.__module__,id(self)))
+            annotateException('\nraised by class %s(%s)@0x%8.8x wrap\n' % (self.__class__.__name__,self.__class__.__module__,id(self)))
         self._H = H
         self._H0 = dims and dims[0][1] or 0
         self._wrapInfo = aW,aH
@@ -1281,7 +1279,7 @@ class _FindSplitterMixin:
         C = content if content is not None else self._content
         for f in C:
             if isinstance(f,ListFlowable):
-                F.extend(self._getContent(f._content))
+                F.extend(f._getContent())
             else:
                 F.append(f)
         return F
@@ -1771,14 +1769,22 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
 
     def wrap(self,aW,aH):
         #here's where we mess with everything
+        self_frame = getattr(self,'_frame',None)
         if aH<self.spaceBefore+self._needed-_FUZZ:
             #we are going straight to the nextTemplate with no attempt to modify the frames
             G = [PageBreak(), self]
             H1 = 0
         else:
+            if not self_frame:
+                #probably in some kind of container
+                from reportlab.platypus.frames import Frame
+                self._frame = Frame(0,0,aW,0x7fffffff,leftPadding=0, rightPadding=0,
+                                    topPadding=0,bottomPadding=0)
             H1, G = self._generated_content(aW,aH)
+            if not self_frame:
+                del self._frame
 
-        self._frame.add_generated_content(*G)
+        if self_frame: self_frame.add_generated_content(*G)
         return 0,min(H1,aH)
 
 class AnchorFlowable(Spacer):
@@ -2056,7 +2062,7 @@ class DDIndenter(Flowable):
                 return self.__dict__[a]
             except KeyError:
                 if a not in ('spaceBefore','spaceAfter'):
-                    raise AttributeError('%r has no attribute %s' % (self,a))
+                    raise AttributeError(f'{self!r} has no attribute {a} dict={self.__dict__}')
         return getattr(self._flowable,a)
 
     def __setattr__(self,a,v):
@@ -2213,6 +2219,7 @@ class ListFlowable(_Container,Flowable):
 
         self._list_content = None
         self._dims = None
+        self._caption = kwds.pop('caption',None)
 
     @property
     def _content(self):
@@ -2403,6 +2410,9 @@ class ListFlowable(_Container,Flowable):
         if spaceAfter is not None:
             f=S[-1]
             f.__dict__['spaceAfter'] = max(f.__dict__.get('spaceAfter',0),spaceAfter)
+
+        if self._caption: S.insert(0,self._caption)
+
         return S
 
 class TopPadder(Flowable):
